/*
	Program cuts off zero level sound file at the end of a file. 
	This is necesarry for combining mp3 decoded pieces of larger audio file
*/
#include <unistd.h>
#include <stdio.h>
#include <sndfile.h>
#include <math.h>

#define MAX_CUT 41
#define MAX_FRAMES 1638400

struct frame {
	short left ;
	short right ;
} ;

int get_max_ampl ( SNDFILE *sndfile, const SF_INFO *sfinfo ) {
	struct frame Frame ;
	int inframes ;
	int max = 0;
	sf_seek ( sndfile, 0 , SEEK_SET ) ;	// Na zacetek	
	inframes = sf_readf_short ( sndfile, (short *)&Frame , 1 ) ;
	while ( inframes >0 ) { 
		if ( Frame.left > max ) {
			max = Frame.left ;
		} ;
		if ( Frame.right > max ) {
			max = Frame.right ;
		} ;
		inframes = sf_readf_short ( sndfile, (short *) &Frame , 1 ) ;
	}
	return max ;
}

int get_last_usable ( int max_cut, SNDFILE *audiofile, const SF_INFO *sfinfo ) {

	off_t position;
	int i,last_usable;
	size_t inframes ;
	struct frame Frame ;	

	position = sf_seek ( audiofile, 0 , SEEK_END ) ;	// na konec datoteke
	i = 1 ;
	last_usable = sfinfo->samples ;
	position = sf_seek ( audiofile, -i , SEEK_END ) ;
	inframes = sf_readf_short ( audiofile, (short *) &Frame , 1 ) ;

	while ( inframes > 0 ) {
		if (( abs(Frame.left) > max_cut ) || ( abs(Frame.right) > max_cut )) {
			break ;
		}
		last_usable -- ;
		i ++ ;
		position = sf_seek ( audiofile, -i , SEEK_END ) ;
		inframes = sf_readf_short ( audiofile, (short *) &Frame , 1 ) ;
	}
	return ( last_usable ) ;
}

int get_first_usable ( int max_cut, SNDFILE *audiofile, const SF_INFO *sfinfo ) {

	off_t position;
	int first_usable;
	size_t inframes ;
	struct frame Frame ;
	
	position = sf_seek ( audiofile, 0 , SEEK_SET ) ;	// na zacetek datoteke
	first_usable = 0 ;
	inframes = sf_readf_short ( audiofile, (short *) &Frame , 1 ) ;
	while ( inframes > 0 ) {
		if (( abs(Frame.left) > max_cut ) || ( abs(Frame.right) > max_cut )) {
			break ;
		}
		first_usable ++ ;
		inframes = sf_readf_short ( audiofile, (short *) &Frame , 1 ) ;
	}	
	return ( first_usable ) ;
}

int main ( int argc, const char ** argv ) {
	char * filename ;
	char outfile[255];
	SNDFILE *audiofile ;
	SNDFILE *newaudiofile ;
	SF_INFO sfinfo ;
	struct frame Frames[MAX_FRAMES] ;
	size_t inframes, outframes ;
	int i, max_cut, first_usable, last_usable , max_ampl, all, written ;
	off_t position ;
	int gain ;
	pid_t mypid ;
	
	max_cut = MAX_CUT ;
	
	if ( argc < 4 ) {
		printf ( "Usage: %s -g GAIN filename\n" , argv[0] ) ;
		return 1;
	}
	if ( argv[2] ) {
		gain = atoi ( (char *) argv[2] ) ;
	} else {
		printf ( "Usage: %s -g GAIN filename\n" , argv[0] ) ;
		return 1;
	}
	
	filename = (char *)argv[3];
	audiofile = sf_open_read ( filename, &sfinfo ) ;
	max_ampl = get_max_ampl ( audiofile , &sfinfo) ;
	max_cut = pow ( 10 , - gain/20 ) * 32767 ; // * max_ampl ;
	first_usable = get_first_usable (max_cut,audiofile, &sfinfo) ;
	fprintf (stderr, "Start at %d\n" , first_usable ) ;
	last_usable = get_last_usable(max_cut,audiofile,&sfinfo) ;	
	fprintf (stderr, "Finish at %d of %d\n" , last_usable, sfinfo.samples ) ;
	mypid = getpid();	
	sprintf ( outfile, "tmp%d.wav" , mypid );
	newaudiofile = sf_open_write ( outfile , &sfinfo ) ;
	position = sf_seek ( audiofile, first_usable , SEEK_SET ) ;
	all = first_usable ;
	written = 0 ;
	fprintf (stderr, "Start at %d\n" , all ) ;
	while ( all < last_usable ) {
		inframes = sf_readf_short ( audiofile, (short *)&Frames , MAX_FRAMES ) ;
		all += inframes ;
		if ( all < last_usable ) {
			outframes = sf_writef_short ( newaudiofile, (short *)&Frames , inframes ) ;
			written += outframes ;
		} else {
			inframes -= ( all - last_usable );
			outframes = sf_writef_short ( newaudiofile, (short *)&Frames , inframes ) ;
			written += outframes ;
			break ;
		}
	}
	fprintf (stderr, "Written: %d = %d\n" , written, last_usable - first_usable ) ;

/*
	for ( i = first_usable ; i < last_usable; i++ ) {
		inframes = sf_readf_short ( audiofile, (short *)&Frame , 1 ) ;
		outframes = sf_writef_short ( newaudiofile, (short *)&Frame , 1 ) ;
	}
*/

	sf_close ( newaudiofile ) ;
	sf_close ( audiofile ) ;
	i = rename ( outfile , filename);
	if ( i != 0 ) {
		fprintf (stderr, "Napaka pri movanju: %d" , i ) ;
	}

	return 0 ;
}
